package Xconfig::initrd;




use lib '/usr/lib/libDrakX';
require bootloader;
require fsedit;
require fs;
use c;
use common;
use interactive;
use log;
use run_program;

sub get_bootloader {
    -x "$::prefix/sbin/display_driver_helper" or return;

    my $all_hds = fsedit::get_hds();
    fs::get_info_from_fstab($all_hds);
    bootloader::read($all_hds);
}

sub handle_initrds {
    my ($bootloader, $rebuild, $o_in) = @_;
    my $curr_kernel = c::kernel_version();

    my %done;
    my @need_rebuild;

    $o_in and my ($wait, $wait_message) = $o_in->wait_message_with_progress_bar(N("Initrd regeneration"));
    $wait_message and $wait_message->(N("Please wait, checking initrds..."), 0, 1);

    foreach (grep { $_->{initrd} } @{$bootloader->{entries}}) {
	my $kernel_str = bootloader::vmlinuz2kernel_str($_->{kernel_or_dev}) or next;

	# in check mode handle current kernel only
	$rebuild or $kernel_str->{version} eq $curr_kernel or next;

	my $initrd_long = '/boot/' . bootloader::kernel_str2initrd_long($kernel_str);
	next if $done{$initrd_long}++ or not -e "$::prefix$initrd_long";

	next if run_program::rooted($::prefix, "/sbin/display_driver_helper", "--check-initrd", "$initrd_long");

	# return if we were only checking
	return 1 if not $rebuild;

	push(@need_rebuild, { entry   => $_,
			      version => $kernel_str->{version},
			      initrd  => $initrd_long });
    }

    scalar @need_rebuild > 0 or return;
    
    my $counter = 0;
    $wait_message and $wait_message->(N("Configuring the display driver... (%d kernels)", scalar @need_rebuild), $counter++, scalar @need_rebuild);

    foreach (@need_rebuild) {
	bootloader::rebuild_initrd($_->{version}, $bootloader, $_->{entry}, $_->{initrd});
	$wait_message and $wait_message->(0, $counter++, scalar @need_rebuild);
    }
}

sub is_rebuild_needed {
    my ($bootloader) = @_;
    $bootloader ||= get_bootloader() or return;
    handle_initrds($bootloader);
}

sub rebuild_initrds {
    my ($bootloader, $o_in) = @_;
    log::l("rebuilding initrds for KMS configuration change");
    $bootloader ||= get_bootloader() or return;
    handle_initrds($bootloader, 1, $o_in);
}

sub rebuild_initrds_if_needed {
    my ($o_in) = @_;
    my $bootloader = get_bootloader() or return;
    is_rebuild_needed($bootloader) or return;
    rebuild_initrds($bootloader, $o_in);
}

1;
